from pydantic import BaseModel

__all__ = [
    'Category',
    'GetCategoriesResponse',
    'Subcategory'
]


class Subcategory(BaseModel):
    id: str
    parent: str
    order: str
    name: str
    name_ru: str
    name_by: str


class Category(BaseModel):
    id: str
    name: str
    name_ru: str
    name_by: str
    version: str
    order: str
    subcategories: list


class GetCategoriesResponse(BaseModel):
    categories: list[Category]



from kufar.base import BaseService
from kufar.utils.auth import auth_required

from . import consts
from .schemas import (
    GetAdvertByListIdResponse,
    GetAdvertResponse,
    GetPhoneResponse,
    GetUserSearchAdvertsResponse,
)


class AdsService(BaseService):
    async def get_advert(self, ad_id: int, lang: str = "ru") -> GetAdvertResponse:
        return await self.client.get(
            consts.GET_ADVERT.format(ad_id=ad_id),
            headers=consts.SEGMENTATION_HEADER,
            params={"lang": lang},
        )

    async def get_advert_by_list_id(
        self, msgi: str, size: str, lang: str = "ru"
    ) -> GetAdvertByListIdResponse:
        return await self.client.get(
            consts.SEARCH_ADVERTS,
            headers=consts.SEGMENTATION_HEADER_WITH_HACK,
            params={"msgi": msgi, "size": size, "lang": lang},
        )

    async def get_ad_phone(self, ad_id: int) -> GetPhoneResponse:
        return await self.client.get(consts.GET_PHONE.format(ad_id=ad_id))

    @auth_required
    async def user_search_adverts(
        self, size: int, cursor: str | None = None
    ) -> GetUserSearchAdvertsResponse:
        return await self.client.get(
            consts.SEARCH_ADVERTS,
            headers=consts.USER_SEGMENTATION_HEADER,
            params={"size": size, "cursor": cursor},
        )

    async def search_adverts(
        self, size: int, query: str, cursor: str | None = None
    ) -> GetAdvertByListIdResponse:
        return await self.client.get(
            consts.SEARCH_ADVERTS,
            headers=consts.SEGMENTATION_HEADER_WITH_HACK,
            params={"query": query, "size": size, "cursor": cursor},
        )



from functools import wraps
from typing import Callable, TypeVar

F = TypeVar('F', bound=Callable)


class UnauthorizedException(Exception):
    def __init__(self):
        super().__init__('method required authorization. You can use method `authenticate`')


def auth_required(f: F) -> F:
    @wraps(f)
    async def wrapper(*args, **kwargs):
        instance = args[0]
        session_id = instance.client.headers.get('session-id')
        token = instance.client.headers.get('authorization')
        if not session_id or not token:
            raise UnauthorizedException()
        return await f(*args, **kwargs)

    return wrapper # type: ignore



from kufar.base import BaseService
from kufar.utils.auth import auth_required

from . import consts
from .schemas import ModifyAdResponse, MyAdsCounterResponse, MyAdsResponse


class UserAdsService(BaseService):
    @auth_required
    async def get_counters(self):
        return await self.client.post_legacy(consts.GET_COUNTERS)

    @auth_required
    async def activate_ad(self, ad_id: int) -> ModifyAdResponse:
        return await self.client.post_legacy(
            consts.ACTIVATE_AD, params={"ad_id": ad_id}
        )

    @auth_required
    async def delete_ad(self, ad_id: int) -> ModifyAdResponse:
        return await self.client.post_legacy(consts.DELETE_AD, params={"ad_id": ad_id})

    @auth_required
    async def get_my_ads(self, offset: int, limit: int, status: str) -> MyAdsResponse:
        return await self.client.get_legacy(
            consts.GET_MY_ADS,
            params={"offset": offset, "limit": limit, "status": status, "stats": 1},
        )

    @auth_required
    async def get_my_ads_count(self) -> MyAdsCounterResponse:
        return await self.client.get_legacy(consts.GET_MY_ADS_COUNT)

    @auth_required
    async def prolong_ad(self, ad_id: int) -> ModifyAdResponse:
        return await self.client.post_legacy(consts.PROLONG_AD, params={"ad_id": ad_id})



from pydantic import BaseModel
from typing import Literal

__all__ = [
    'Advert',
    'Image',
    'Page',
    'Pagination',
    'GetAdvertResponse',
    'Parameter',
    'GetPhoneResponse',
    'PaidService',
    'GetAdvertByListIdResponse',
    'GetUserSearchAdvertsResponse'
]


class Image(BaseModel):
    id: str
    media_storage: str
    path: str
    yams_storage: bool


class PaidService(BaseModel):
    highlight: bool
    polepos: bool
    ribbons: str
    halva: bool


class Parameter(BaseModel):
    name_label: str
    value_label: str
    name: str
    value: str


class Advert(BaseModel):
    ad_id: int
    account_id: int
    images: list[Image]
    account_parameters: list[Parameter]
    ad_parameters: list[Parameter]
    list_time: int | None = None
    paid_services: list[PaidService] | None = None
    list_id: int | None = None
    message_id: str | None = None
    price_usd: int | None = None
    price_eur: int | None = None
    price_byn: int | None = None
    remuneration_type: str | None = None
    company_ad: bool | None = None
    phone: str | None = None
    category: str | None = None
    subject: str | None = None
    body: str | None = None
    type: str | None = None
    ad_link: str | None = None
    phone_hidden: bool | None = None
    currency: Literal['BYN', 'USD'] | None = None


class GetAdvertResponse(BaseModel):
    result: Advert


class GetAdvertByListIdResponse(BaseModel):
    ads: list[Advert]


class GetPhoneResponse(BaseModel):
    phone: str


class Page(BaseModel):
    label: Literal['next', 'prev']
    num: int
    token: str


class Pagination(BaseModel):
    pages: list[Page]


class GetUserSearchAdvertsResponse(BaseModel):
    total: int
    ads: list[Advert]
    pagination: Pagination



ADD_FAVORITE_ADVERT = '/account_save_ad.json'
# @GET
# @PAYLOAD
# list_id - QUERY - int
# @RESPONSE - ToggleFavorite

DELETE_FAVORITE_ADVERT = '/account_delete_save_ad.json'
# @GET
# @PAYLOAD
# list_id - QUERY - int
# @RESPONSE - ToggleFavorite


GET_FAVORITE_ADVERTS = '/account_get_list_saved_ads.json'
# @GET
# @RESPONSE - FavoriteAdverts


GET_FAVORITE_ADVERTS_IDS = '/account_get_favourite_ads_ad_ids.json'
# @GET
# RESPONSE: {"ad_ids": list[int]}




LOG_IN = '/v2/auth/signin'
LOG_IN_APPLE = '/v2/auth/signin/apple'
LOG_IN_GOOGLE = '/v2/auth/signin/google'
LOG_IN_MAIL_RU = '/v2/auth/signin/mailru'
LOG_IN_YANDEX = '/v2/auth/signin/yandex'
LOG_OUT = '/v1/auth/signout'
LOG_OUT_ALL = '/v1/auth/signout/all'
CHANGE_PASSWORD = '/v1/auth/change_password'
CREATE_ACCOUNT = '/v2/auth/signup'
IS_LOGIN_CAPTCHA_REQUIRED = '/v2/auth/signin/recaptcha_required'
LOGIN_HISTORY = '/v2/auth/signin/history'
RESET_PASSWORD = '/v1/auth/reset_password'
RESET_PASSWORD_REQUEST = '/v1/auth/reset_password_request'
REQUEST_SMS_CODE = '/v2/auth/phone/code'
VERIFY_SMS_CODE = '/v2/auth/phone/code'



from datetime import datetime
from enum import Enum

from pydantic import BaseModel

__all__ = [
    'MyAd',
    'ModifyAdResponse',
    'Thumb',
    'MyAdStatistics',
    'MyAdsCounterResponse',
    'AdStatusEnum',
    'MyAdsCounter',
    'MyAdsResponse',
]


class ModifyAdResponse(BaseModel):
    status: str
    success_header: str
    success_description: str
    success_ok_button: str
    error_header: str
    error_description: str
    error_ok_button: str
    error_buy_button: str
    error_label: str


class MyAdsCounter(BaseModel):
    status: str
    value: int


class MyAdsCounterResponse(BaseModel):
    counters: list[MyAdsCounter]
    status: str


class AdStatusEnum(Enum):
    PENDING_REVIEW = 'PENDING_REVIEW'
    DEACTIVATED = 'DEACTIVATED'
    INACTIVE = 'INACTIVE'
    PENDING_SLOT = 'PENDING_SLOT'
    REFUSED = 'REFUSED'
    ACTIVE = 'ACTIVE'
    PENDING_REVIEW_AFTER_EDIT = 'PENDING_REVIEW_AFTER_EDIT'
    PENDING_REVIEW_AFTER_ACTIVATION = 'PENDING_REVIEW_AFTER_ACTIVATION'
    DELETED = 'DELETED'
    HIDDEN = 'HIDDEN'


class MyAdStatistics(BaseModel):
    view: int
    phone: int
    favorite: int


class Thumb(BaseModel):
    image_id: str
    media_storage: str
    path: str
    yams_storage: bool


class MyAd(BaseModel):
    ad_id: int
    list_id: int | None = None
    action_id: int
    ad_status: AdStatusEnum
    db_status: AdStatusEnum
    body: str | None = None
    category: int
    category_name: str
    category_parent: int
    category_parent_name: str
    company_ad: bool
    currency: str
    date: datetime | None = None
    date_expired: datetime | None = None
    delete_from_date: datetime | None = None
    highlight: bool
    phone_hidden: bool
    polepos: bool
    safedeal_enabled: bool
    price: int | None = None
    price_suffix: str | None = None
    prolong_perm: bool
    ribbons: str | None = None
    subject: str | None = None
    remuneration_type: str | None = None
    thumbs: list[Thumb]
    statistics: MyAdStatistics | None = None
    delivery_enabled: bool
    halva: bool
    delete_perm: bool
    region: int
    region_name: str | None = None
    activate_perm: bool
    is_favourite: bool
    link: str | None = None
    polepos_end: datetime | None = None
    highlight_end: datetime | None = None
    ribbons_end: datetime | None = None


class MyAdsResponse(BaseModel):
    status: str
    total: int
    numperpage: int
    ads: list[MyAd]



MARK_SEARCH_SEEN = '/saved-search/v1/accounts/{account_id}/searches/{search_id}/views'
DELETE_SEARCH = '/saved-search/v1/accounts/{account_id}/searches/{search_id}'
GET_FAVORITE_SEARCHES = '/saved-search/v1/accounts/{account_id}/searches'
SAVE_SEARCH = '/saved-search/v1/accounts/{account_id}/searches'



from datetime import datetime

from pydantic import BaseModel

__all__ = [
    'Account',
    'GetCurrentAccountResponse',
    'Profile'
]


class Account(BaseModel):
    account_id: int
    email: str
    created_at: datetime
    phone: str
    verified_phone: bool


class ProfileParams(BaseModel):
    company_ad: int
    date_of_birth: datetime | None = None
    gdpr_confirm: int
    gdpr_confirm_date: datetime
    gdpr_confirm_version: int
    gender: str | None = None
    name: str
    origin: str
    phone_hidden: int
    profile_image: str


class ProfileReputationFeedback(BaseModel):
    overallScore: int
    receivedCount: int


class ProfileReputation(BaseModel):
    feedback: ProfileReputationFeedback


class ProfileBase(BaseModel):
    reputation: ProfileReputation


class ProfileFollowings(BaseModel):
    followers: int
    followings: int


class Profile(BaseModel):
    params: ProfileParams
    profile: ProfileBase
    followings: ProfileFollowings


class GetCurrentAccountResponse(BaseModel):
    account: Account
    profile: Profile



GET_ADVERT = '/search-api/v1/ads/{ad_id}/rendered'
SEARCH_ADVERTS = '/search-api/v1/search/rendered-paginated'
GET_PHONE = '/search-api/v1/ads/{ad_id}/phone'


GEO_ADVERTS = '/search-api/v1/search/map'
# @GET
# size - QUERY - str
# RESPONSE -> GeoAdverts

GET_ADVERTS_COUNT = '/search-api/v1/search/count'
# GET
# RESPONSE -> Count

GET_PHONE_CAPTCHA = '/search-api/v1/ads/{ad_id}/phone/captcha'
# GET
# HEADER - g-recaptcha-response
# RESPONSE -> Phone


# SEARCH_ADVERTS
# GET
# size - QUERY - int
# cursor - QUERY - str
# lang - QUERY - str
# query - QUERY - str
# RESPONSE -> Adverts

SEGMENTATION_HEADER = {
    "X-Segmentation": "routing=android_generalist;application=ad_view;platform=android"
}
USER_SEGMENTATION_HEADER = {
    "X-Segmentation": "routing=android_generalist;application=ad_listing;platform=android"
}
SEGMENTATION_HEADER_WITH_HACK = {
    **SEGMENTATION_HEADER,
    "X-MCHack": "true"
}



from pydantic import BaseModel, Field
from typing import Literal

__all__ = [
    'Session',
    'LogInResponse',
    'LogInPayload',
    'ChangePasswordPayload',
    'HistoryResponse',
    'IsCaptchaRequiredResponse',
    'RequestSmsCodeResponse',
    'ResetPasswordPayload',
    'ResetPasswordRequestPayload',
    'SignUpPayload',
    'VerifySmsCodePayload'
]


class LogInPayload(BaseModel):
    login: str
    password: str
    captcha_user_response: str | None = None


class LogInResponse(BaseModel):
    session_id: str = Field(validation_alias='session-id')
    jwt: str


class ChangePasswordPayload(BaseModel):
    email: str
    current_password: str
    password: str


class SignUpPayload(BaseModel):
    email: str
    password: str
    password_confirm: str
    company_ad: bool
    captcha_user_response: str
    captcha_platform: Literal['android']
    gdpr_confirm: bool


class IsCaptchaRequiredResponse(BaseModel):
    is_required: bool


class Session(BaseModel):
    is_active: bool
    created_at: str
    city_name: str
    country_name: str
    country_code: str
    client_ip: str
    user_agent: str
    device_type: str


class HistoryResponse(BaseModel):
    history: list[Session]


class ResetPasswordPayload(BaseModel):
    password: str
    password_confirm: str
    activation_code: str


class ResetPasswordRequestPayload(BaseModel):
    email: str


class RequestSmsCodeResponse(BaseModel):
    code_token: str


class VerifySmsCodePayload(BaseModel):
    code_token: str
    code: str



import json
import os

import aiofiles


class State:
    __state: dict[str, str | int]
    __file_path: str

    def __init__(self, file_path: str):
        self.__file_path = file_path
        self.__state = {}

    async def load(self) -> None:
        if os.path.isfile(self.__file_path):
            async with aiofiles.open(self.__file_path, "r") as f:
                self.__state = json.loads(await f.read())

    async def save(self) -> None:
        async with aiofiles.open(self.__file_path, "w") as f:
            await f.write(json.dumps(self.__state))

    def __getattr__(self, name: str) -> str | int | None:
        return object.__getattribute__(self, "_State__state").get(name, None)

    def __setattr__(self, name: str, value: str | int) -> None:
        if name in self.__annotations__.keys():
            return object.__setattr__(self, name, value)
        object.__getattribute__(self, "_State__state")[name] = value



import hashlib

SECRET = '49f6882c2234a1ec7f9acedb849852cbad7aa7e1'

def decode_signature(input_str: str):
    input_str = input_str + SECRET
    message_digest = hashlib.sha1()
    input_bytes = input_str.encode('utf-8')
    message_digest.update(input_bytes)
    hashed_str = message_digest.hexdigest()
    return hashed_str



ACTIVATE_AD = '/account_activate_ad.json'
DELETE_AD = '/account_delete_ad.json'
GET_MY_ADS = '/account_get_published_ads.json'
GET_MY_ADS_COUNT = '/account_get_published_ads_count.json'
PROLONG_AD = '/account_prolong_ad.json'
GET_COUNTERS = '/account_get_counters.json'


from pydantic import BaseModel

__all__ = [
    'SavedSearch',
    'Counters',
    'AutoNames',
    'SavedSearches'
]


class AutoNames(BaseModel):
    by: str
    ru: str


class Counters(BaseModel):
    new: int | None = None
    total: int | None = None


class SavedSearch(BaseModel):
    auto_names: AutoNames | None = None
    query: str | None = None
    counters: Counters | None = None
    created_at: int | None = None


class SavedSearches(BaseModel):
    items: list[SavedSearch]
    cursor: str | None = None



from .account import *
from .ads import *
from .auth import *
from .categories import *
from .favorites import *
from .filters import *
from .saved import *
from .user_ads import *
from .messaging import *



import inspect
from dataclasses import dataclass
from functools import wraps

from pydantic import BaseModel, ValidationError
from kufar.utils.requester import Requester


@dataclass
class BaseService:
    client: Requester
    base_url: str

    def __getattribute__(self, item: str):
        attr = super().__getattribute__(item)

        if callable(attr):
            return_type = inspect.signature(attr).return_annotation

            if not return_type:
                return attr

            if inspect.isclass(return_type) and issubclass(return_type, BaseModel):

                @wraps(attr)
                async def wrapper_func(*args, **kwargs):
                    self.client.url = self.base_url
                    response = await attr(*args, **kwargs)
                    try:
                        return return_type.model_validate(response)
                    except ValidationError as e:
                        print(
                            f"Error while validating response. Response is: {response}\nError is: {e}"
                        )
                        return None

                return wrapper_func
        return attr



GET_FILTERS = "/taxonomy-proxy/v1/dispatch"

_BASE_PARAMS = {"routing": "android_generalist", "taxonomy-version": "2"}
BASE_FILTER_PARAMS = {
    "application": "ad_listing_base",
    "platform": "android",
    **_BASE_PARAMS,
}
FILTERS_PARAMS = {"application": "ad_listing", "platform": "android", **_BASE_PARAMS}



from kufar.base.service import BaseService
from kufar.utils.auth import auth_required

from . import consts
from .schemas import UnreadMessagesCountResponse


class MessagingService(BaseService):
    @auth_required
    async def unread_count(self) -> UnreadMessagesCountResponse:
        return await self.client.get(consts.MESSAGES_COUNTER)



from .base import consts
from .packages import (
    AccountService,
    AdsService,
    AuthService,
    CategoryService,
    LogInPayload,
    MessagingService,
    SavedService,
    UserAdsService,
    FilterService,
)
from .utils.requester import Requester
from .utils.state import State


class KufarApi:
    __inited: bool = False

    _client: Requester

    account: AccountService
    ads: AdsService
    auth: AuthService
    categories: CategoryService
    saved: SavedService
    user_ads: UserAdsService
    messaging: MessagingService
    filters: FilterService

    state: State

    def __init__(self, client: Requester, state: State):
        self._client = client

        self.state = state

        self.ads = AdsService(self._client, consts.API_URL)
        self.auth = AuthService(self._client, consts.CRE_API_URL)
        self.categories = CategoryService(self._client, consts.API_URL)
        self.saved = SavedService(self._client, consts.API_URL)
        self.user_ads = UserAdsService(self._client, consts.BAPI_URL)
        self.account = AccountService(self._client, consts.API_URL)
        self.messaging = MessagingService(self._client, consts.API_URL)
        self.filters = FilterService(self._client, consts.API_URL)

    def _load_tokens_from_state(self):
        if self.state.access_token is not None:
            self._client.patch_headers(
                {
                    "authorization": f"Bearer {self.state.access_token}",
                    "session-id": str(self.state.session_id),
                }
            )

    async def init(self):
        if self.__inited:
            return
        await self.state.load()
        self._load_tokens_from_state()

        await self.filters.get_base_filters()

        self.__inited = True

    async def authenticate(self, email: str, password: str):
        if (await self.auth.is_login_captcha_required()).is_required:
            raise Exception("Captcha required")
        tokens = await self.auth.login(LogInPayload(login=email, password=password))
        if tokens:
            self.state.access_token = tokens.jwt
            self.state.session_id = tokens.session_id
            await self.state.save()
            self._load_tokens_from_state()



from typing import Literal

from pydantic import BaseModel


class ParamsRefValue(BaseModel):
    value: str
    labels: dict[Literal["ru", "by"], str]
    hint: str | None = None
    image_url: str | None = None


class ParamsRefUIComponent(BaseModel):
    fallback_type: str | None = None
    type: str | None = None


class LanguageRef(BaseModel):
    ru: str
    by: str


class RangeRef(BaseModel):
    default_lower: int
    default_upper: int
    lower: int
    step: int
    upper: int


class ParamsRef(BaseModel):
    variation_id: int
    name: str
    url_name: str
    required: bool
    type: str
    meta: str
    multi: bool
    ui_component: ParamsRefUIComponent | None = None
    # actions: any
    labels: dict[str, dict[Literal["by", "ru"], str]]
    values: list[ParamsRefValue] | None = None
    external_values_url: str | None = None
    range: RangeRef | None = None
    min_taxonomy_version: int
    hint: LanguageRef | None = None
    image_url: str | None = None
    is_type: bool


class ParamsRule(BaseModel):
    rule: dict[str, str]
    refs: list[str]


class ParametersOrder(BaseModel):
    dispatch_type: Literal["many", "one"]
    deduplication_key: Literal["name"] | None = None
    refs: dict[str, ParamsRef]
    rules: list[ParamsRule]


class ParamsMetadata(BaseModel):
    parameters: ParametersOrder


class MetadataResponse(BaseModel):
    metadata: ParamsMetadata



from kufar.base import BaseService
from kufar.utils.auth import auth_required

from . import consts
from .schemas import (
    ChangePasswordPayload,
    HistoryResponse,
    IsCaptchaRequiredResponse,
    LogInPayload,
    LogInResponse,
    RequestSmsCodeResponse,
    ResetPasswordPayload,
    ResetPasswordRequestPayload,
    SignUpPayload,
    VerifySmsCodePayload,
)


class AuthService(BaseService):
    async def login(self, payload: LogInPayload) -> LogInResponse:
        return await self.client.post(
            consts.LOG_IN,
            {
                **payload.model_dump(exclude_none=True),
                "captcha_platform": "android",
                "captcha_type": "recaptcha",
                "captcha_secret_version": "v1",
            },
            params={"token_type": "user"},
        )

    async def login_apple(self, token: str, full_name: str) -> LogInResponse:
        return await self.client.post(
            consts.LOG_IN_APPLE, {"token": token, "full_name": full_name}
        )

    async def login_google(self, token: str) -> LogInResponse:
        return await self.client.post(consts.LOG_IN_GOOGLE, {"token": token})

    async def login_mail_ru(self, token: str) -> LogInResponse:
        return await self.client.post(consts.LOG_IN_MAIL_RU, {"token": token})

    async def login_yandex(self, token: str) -> LogInResponse:
        return await self.client.post(consts.LOG_IN_YANDEX, {"token": token})

    @auth_required
    async def logout(self) -> None:
        return await self.client.get(consts.LOG_OUT)

    @auth_required
    async def logout_all(self) -> None:
        return await self.client.get(consts.LOG_OUT_ALL)

    async def change_password(self, payload: ChangePasswordPayload) -> any:
        return await self.client.post(consts.CHANGE_PASSWORD, payload.model_dump())

    async def create_account(self, payload: SignUpPayload) -> any:
        return await self.client.post(consts.CREATE_ACCOUNT, payload.model_dump())

    async def is_login_captcha_required(self) -> IsCaptchaRequiredResponse:
        return await self.client.get(consts.IS_LOGIN_CAPTCHA_REQUIRED)

    @auth_required
    async def login_history(self, cursor: str | None = None) -> HistoryResponse:
        return await self.client.get(consts.LOGIN_HISTORY, params={"cursor": cursor})

    async def reset_password(self, payload: ResetPasswordPayload) -> any:
        return await self.client.post(consts.RESET_PASSWORD, payload.model_dump())

    async def reset_password_request(self, payload: ResetPasswordRequestPayload) -> any:
        return await self.client.post(
            consts.RESET_PASSWORD_REQUEST, payload.model_dump()
        )

    async def request_sms_code(self, phone: str) -> RequestSmsCodeResponse:
        return await self.client.get(consts.REQUEST_SMS_CODE, params={"phone": phone})

    async def verify_sms_code(self, payload: VerifySmsCodePayload) -> LogInResponse:
        return await self.client.post(consts.VERIFY_SMS_CODE, payload.model_dump())



from kufar.base.service import BaseService
from .schemas import MetadataResponse
from . import consts


class FilterService(BaseService):
    state: dict[str, dict]

    async def get_base_filters(self) -> MetadataResponse:
        return await self.client.get(
            consts.GET_FILTERS, params=consts.BASE_FILTER_PARAMS
        )

    async def get_filters(self, parent: int) -> MetadataResponse:
        return await self.client.get(
            consts.GET_FILTERS,
            params={"parent": parent, **consts.FILTERS_PARAMS},
        )

    async def load_filters(self):
        _filters = await self.get_base_filters()
        # There are should be something that saving to the state cities, categories and other
        # important things that respond from this method



from json import JSONDecodeError
from typing import Literal, Self

from aiohttp import ClientSession, ContentTypeError

from .signature_decoder import decode_signature
from ..base import consts


class Requester:
    __base_url: str
    __client: ClientSession | None = None
    __headers: dict[str, str]
    __hash: str = ""

    def __init__(self):
        self.__base_url = ""
        self.__headers = consts.BASE_HEADERS

    async def __aenter__(self) -> Self:
        self.__client = ClientSession(
            base_url=self.__base_url if len(self.__base_url) else None,
            headers=self.__headers,
        )
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.__client:
            await self.__client.close()
        self.__client = None

    @property
    def url(self):
        return self.__base_url

    @url.setter
    def url(self, value: str):
        self.__base_url = value

    async def post(self, path: str, payload: dict | None = None, **kwargs):
        prev_headers = kwargs.get("headers") or {}
        kwargs.update(
            {
                "headers": {
                    **prev_headers,
                    "content-type": "application/json; charset=UTF-8",
                }
            }
        )
        return await self._request("POST", path, json=payload, **kwargs)

    async def get(self, path: str, **kwargs):
        return await self._request("GET", path, **kwargs)

    async def delete(self, path: str, **kwargs):
        return await self._request("DELETE", path, **kwargs)

    async def post_legacy(self, path: str, **kwargs):
        return await self._legacy_request("POST", path, **kwargs)

    async def get_legacy(self, path: str, **kwargs):
        return await self._legacy_request("GET", path, **kwargs)

    @property
    def headers(self):
        return self.__headers

    def patch_headers(self, new_headers: dict[str, str]):
        self.__headers = {**self.__headers, **new_headers}
        if self.__client:
            self.__client.headers.update(self.__headers)

    def select_headers(self, *args: str):
        if len(args) == 1:
            return self.__headers.get(args[0])
        return {k: v for k, v in self.__headers.items() if k in args}

    async def _request(
        self, method: Literal["GET", "POST", "PUT", "DELETE"], path: str, **kwargs
    ):
        if not self.__client:
            raise Exception(
                "Not initialized client. You should use async context manager"
            )
        if not self.__base_url:
            raise Exception("base_url not specified")
        if kwargs.get("headers"):
            kwargs["headers"] = {
                **self.__headers,
                **kwargs.get("headers"),  # type: ignore
            }
        else:
            kwargs["headers"] = self.__headers
        url = f"{self.__base_url}{path}"
        params = exclude_none(kwargs.get("params") or {})
        kwargs.pop("params")
        async with self.__client.request(method, url, **kwargs, params=params) as res:
            try:
                return await res.json()
            except (ContentTypeError, JSONDecodeError) as e:
                print(f"Error while decoding. Content: {await res.text()}")
                raise e

    async def _legacy_request(self, method: str, path: str, **kwargs) -> any:  # type: ignore
        url = f"{self.__base_url}{path}"
        if not self.__client:
            return None
        headers: dict = (
            self.select_headers(
                "user-agent", "x-app-name", "x-app-version", "x-device-id"
            )
            or {}
        )  # type: ignore
        async with self.__client.request(
            method,
            url,
            **kwargs,
            allow_redirects=False,
            params={
                "app_id": "android",
                "account_token": self.select_headers("session-id"),
                "hash": decode_signature(self.__hash),
                **(exclude_none(kwargs.get("params") or {})),
            },
            headers={**headers, **(kwargs.get("headers") or {})},
        ) as res:
            try:
                data = await res.json()
            except (ContentTypeError, JSONDecodeError) as e:
                print(f"Error while decoding. Content: {(await res.text())[:1000]}")
                raise e
        if "authorize" in data:
            self.__hash = str(data.get("authorize").get("challenge", ""))
            return await self._legacy_request(method, path)
        return data


def exclude_none(input: dict) -> dict:
    return {k: v for k, v in input.items() if v is not None}



from kufar.base import BaseService
from . import consts
from .schemas import SavedSearch, SavedSearches


class SavedService(BaseService):
    async def mark_search_seen(self, account_id: int, search_id: int) -> None:
        return await self.client.post(
            consts.MARK_SEARCH_SEEN.format(account_id=account_id, search_id=search_id)
        )

    async def delete_search(self, account_id: int, search_id: int) -> None:
        return await self.client.delete(
            consts.DELETE_SEARCH.format(account_id=account_id, search_id=search_id)
        )

    async def get_searches(
        self, account_id: int, cursor: str | None = None
    ) -> SavedSearches:
        return await self.client.get(
            consts.GET_FAVORITE_SEARCHES.format(account_id=account_id),
            params={"cursor": cursor},
        )

    async def save_search(self, account_id: int, query: str) -> SavedSearch:
        return await self.client.post(
            consts.SAVE_SEARCH.format(account_id=account_id), {"query": query}
        )



from kufar.base import BaseService
from ..account.schemas import GetCurrentAccountResponse
from kufar.utils.auth import auth_required

from . import consts


class AccountService(BaseService):
    @auth_required
    async def get_current_account(self) -> GetCurrentAccountResponse:
        return await self.client.get(consts.GET_CURRENT_ACCOUNT)



from . import consts
from .schemas import GetCategoriesResponse

from kufar.base import BaseService


class CategoryService(BaseService):
    async def get_categories(self) -> GetCategoriesResponse:
        return await self.client.get(consts.GET_CATEGORIES)



from .packages.auth.schemas import *
from .packages.ads.schemas import *
from .packages.saved.schemas import *
from .packages.filters.schemas import *
from .packages.account.schemas import *
from .packages.user_ads.schemas import *
from .packages.categories.schemas import *
from .packages.messaging.schemas import *



API_URL = 'https://api.kufar.by'
CRE_API_URL = 'https://cre-auth.kufar.by'
BAPI_URL = 'https://bapi.kufar.by/api'

BASE_HEADERS = {
    'user-agent': 'Kufar/v3.0.1/13/Xiaomi/Mi 9T Pro',
    'x-app-name': 'Android Kufar',
    'x-app-version': 'v3.0.1',
    'x-device-id': '45f44584-588f-4a92-959e-ad3416a88c28'
}